@@ -55,13 +55,11 @@ class Ec2:
         return image_id
 
     def get_optimal_instance_type(self, options):
-        import logging
-        logging.getLogger("urllib3").setLevel(logging.ERROR)
-
         memory = options['memory']
         cpu = options['cpu']
         usage_class = options['usage_class']
         is_instance_storage_supported = options.get('is_instance_storage_supported')
+        max_spot_interruption_frequency = options.get('max_spot_interruption_frequency')
 
         get_best_instance_types_input = {
             # Required.
@@ -78,6 +76,9 @@ class Ec2:
             'is_best_price': True,
         }
 
+        if usage_class == 'spot' and max_spot_interruption_frequency is not None:
+            get_best_instance_types_input['max_interruption_frequency'] = max_spot_interruption_frequency
+
         if is_instance_storage_supported is not None:
             get_best_instance_types_input['is_instance_storage_supported'] = is_instance_storage_supported
 
