@@ -14,11 +14,9 @@ from .ec2 import Ec2
 
 
 class ContainerLauncher:
-    __LOCAL_MOUNT_PATH = '/mnt/fs'
-
     __ASG_DRAINING_TIMEOUT_IN_MINUTES = 5
     __INSTANCE_TERMINATION_TIMEOUT_IN_MINUTES = 5
-    __SPOT_INSTANCE_TERMINATION_TIMEOUT_IN_MINUTES = 25
+    __SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES = 10
     __ECS_CLUSTER_ATTACHMENTS_UPDATE_TIMEOUT_IN_MINUTES = 5
     __ASG_FINISHING_SCALING_ACTIVITIES_TIMEOUT_IN_MINUTES = 5
     __EC2_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES = 5
@@ -85,11 +83,19 @@ class ContainerLauncher:
                                        options['log_level'] if options and options.get('log_level') else logging.INFO)
             logging.basicConfig(level=log_level)
 
+        if options is not None and 'spot_instance_deploying_timeout_in_minutes' in options:
+            self.__SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES = options.get('spot_instance_deploying_timeout_in_minutes')
+
     def start_container(self, container_input):
         entry_id = container_input.get('id', str(uuid.uuid4()).replace('-', ''))
         ec2_sg_ids = container_input.get('ec2_sg_ids')
         iam_ecs_task_policy_documents = None
         iam_ec2_policy_documents = None
+        iam_ec2_managed_policy_arns = []
+        max_spot_interruption_frequency = 10
+        if container_input.get('spot') is not None:
+            if container_input['spot'].get('max_interruption_frequency') is not None:
+                max_spot_interruption_frequency = container_input['spot']['max_interruption_frequency']
         if container_input.get('iam') is not None:
             if container_input['iam'].get('ecs_task') is not None:
                 if container_input['iam']['ecs_task'].get('policy_documents'):
@@ -97,6 +103,8 @@ class ContainerLauncher:
             if container_input['iam'].get('ec2') is not None:
                 if container_input['iam']['ec2'].get('policy_documents'):
                     iam_ec2_policy_documents = container_input['iam']['ec2']['policy_documents']
+                if container_input['iam']['ec2'].get('managed_policy_arns'):
+                    iam_ec2_managed_policy_arns = container_input['iam']['ec2']['managed_policy_arns']
         try:
             self.__options['name_suffix'] = container_input.get('name_suffix', str(uuid.uuid4()).replace('-', ''))
 
@@ -130,14 +138,17 @@ class ContainerLauncher:
                 get_optimal_instance_type_input = {
                     'cpu': container_input['cpu'],
                     'memory': container_input['memory'],
-                    'usage_class': usage_class
+                    'usage_class': usage_class,
+                    'max_spot_interruption_frequency': max_spot_interruption_frequency
                 }
-                ssd_instance_stores = [storage for storage in storages if storage['type'] == 'ssd_instance_store']
-                if len(ssd_instance_stores) > 0:
-                    get_optimal_instance_type_input['is_instance_storage_supported'] = True
+                if storages is not None:
+                    ssd_instance_stores = [storage for storage in storages if storage['type'] == 'ssd_instance_store']
+                    if len(ssd_instance_stores) > 0:
+                        get_optimal_instance_type_input['is_instance_storage_supported'] = True
 
-                instance_type = ec2.get_optimal_instance_type(get_optimal_instance_type_input)
-                self.logger.debug(f'The optimal instance type is {instance_type}')
+                ec2_instance = ec2.get_optimal_instance_type(get_optimal_instance_type_input)
+                self.logger.debug(f'The optimal instance type is {str(ec2_instance)}')
+                instance_type = ec2_instance['instance_type']
 
             cpu = self.__get_cpu_for_instance(instance_type)
 
@@ -150,7 +161,7 @@ class ContainerLauncher:
             }).get_image_id({'instance_type': instance_type})
             subnet_ids = container_input['subnet_ids']
 
-            ec2_iam_role_arn = self.__create_ec2_iam_role(iam_ec2_policy_documents)
+            ec2_iam_role_arn = self.__create_ec2_iam_role(iam_ec2_managed_policy_arns, iam_ec2_policy_documents)
 
             self.__create_ecs_task_iam_role(iam_ecs_task_policy_documents)
             self.__create_ecs_task_execution_iam_role()
@@ -301,7 +312,7 @@ class ContainerLauncher:
         # Sleep to make the instance profile available
         time.sleep(15)
 
-    def __create_ec2_iam_role(self, policy_documents=None):
+    def __create_ec2_iam_role(self, iam_ec2_managed_policy_arns, policy_documents=None):
         name_suffix = self.__options['name_suffix']
         role_name = f'{self.__name_prefix}-ec2-role-{name_suffix}'
 
@@ -327,12 +338,16 @@ class ContainerLauncher:
 
         self.logger.debug(f'The EC2 IAM role {role_name} has been created')
 
-        for managed_policy_arn in self.__IAM_EC2_MANAGED_POLICY_ARNS:
+        managed_policy_arns = self.__IAM_EC2_MANAGED_POLICY_ARNS + iam_ec2_managed_policy_arns
+
+        for managed_policy_arn in managed_policy_arns:
             self.clients['iam'].attach_role_policy(
                 RoleName=role_name,
                 PolicyArn=managed_policy_arn
             )
 
+        self.__resources['ec2_iam_role_managed_policy_arns'] = managed_policy_arns
+
         self.__resources['ec2_iam_role_policy_document_names'] = []
 
         if policy_documents is not None:
@@ -441,7 +456,7 @@ class ContainerLauncher:
         except Exception as e:
             self.logger.warning(f'An error occurred during the remove_role_from_instance_profile operation: {str(e)}')
         try:
-            for managed_policy_arn in self.__IAM_EC2_MANAGED_POLICY_ARNS:
+            for managed_policy_arn in self.__resources['ec2_iam_role_managed_policy_arns']:
                 self.clients['iam'].detach_role_policy(
                     RoleName=self.__resources.get('ec2_iam_role_name'),
                     PolicyArn=managed_policy_arn
@@ -939,14 +954,14 @@ class ContainerLauncher:
 
         self.logger.info('Waiting for an EC2 instance deploying')
 
-        timeout_in_minutes = self.__SPOT_INSTANCE_TERMINATION_TIMEOUT_IN_MINUTES if is_spot else self.__INSTANCE_TERMINATION_TIMEOUT_IN_MINUTES
+        timeout_in_minutes = self.__SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES if is_spot else self.__EC2_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES
 
         while True:
             now_time = datetime.datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > timeout_in_minutes:
-                raise Exception('EC2 instance deploying timeout')
+                raise Exception(f'EC2 instance deploying timeout ({timeout_in_minutes} minutes)')
             if len(instances) > 0:
                 self.logger.debug('The EC2 instance ID: ' + instances[0]['InstanceId'])
                 response = self.clients['ec2'].describe_instance_status(InstanceIds=[instances[0]['InstanceId']])
